function [bestX, bestFx] = scout(XLow, XHi, InitStep, FinStep, MaxIters,...
                                 MaxPop, MaxScoutPop, bSemiLogDecay, ...
                                 bTrace)
% The SCOUT function implements a simpler version of the Fireworks
% Algorithm.
% This version uses both uniform and normal RNG to perform perturbation on
% the scout local search.
%
% INPUT
% =====
% XLow - array of lower limits of trust region.
% XHi - array of upper limits of trust region.
% InitStep - initial search step size.
% FinStep - final search step size.
% MaxIters - maximum number of iterations.
% MaxPop - the population size.
% MaxScoutPop - the size of the local scout population.
% bSemiLogDecay - flag used to use log-linear step decay (when true) or
% log-log step decayscout (when false).
% bTrace - optional Boolean flag used to trace intermediate results.
%
% OUTPUT
% ======
% bestX - array of best solution.
% bestFx - best function value.
%
%
% global tData
% global yData

fx = @rkf5;
tpc = fix(MaxIters/10); % calculate then % of total iterations
n = 2; % variables are A0 and k
m = n + 1;
pop = zeros(MaxPop,m);
for i=1:MaxPop
  pop(i,1:n) = XLow + (XHi - XLow).*rand(1,n);
  pop(i,m) = fx(pop(i,1:n));
end
pop = sortrows(pop, m);
%------------------------------------------------------------
scoutPop = zeros(MaxScoutPop,m);
scoutPop(:,m) = 1e+99;
if bTrace
  fprintf('%f ', pop(1,1:n));
  fprintf('%e\n', pop(1,m));
end
StepFactor = zeros(1,n);
for iter = 1:MaxIters
  % StepFactor = InitStep + (FinStep - InitStep)*(iter-1)/(MaxIters-1);
  if bSemiLogDecay
    for i=1:n
      StepFactor(i) = exp(log(InitStep(i)) + (iter-1)/(MaxIters-1) * log(FinStep(i)/InitStep(i)));
    end
  else
    for i=1:n
       StepFactor(i) = exp(log(InitStep(i)) + log(FinStep(i)/InitStep(i))*log(iter)/log(MaxIters));
    end
  end
  
  % update main population, whenevr possible
  popMember = zeros(1,m);
  for i=2:MaxPop
    if rand > 0.5
      sf = StepFactor;
      if rand > 0.5, sf = [1 1]; end
      popMember(1:n) = pop(i,1:n) + (pop(1,1:n) - pop(i,1:n)) .* rand(1,n) .* sf(1:n);
    else
      sf = StepFactor;
      if rand > 0.5, sf = [1 1]; end
      popMember(1:n) = XLow + (XHi - XLow) .* rand(1,n) .* sf(1:n);
    end
    popMember(m) = fx(popMember(1:n));
    if popMember(m) < pop(i,m)
      pop(i,:) = popMember;
    end
  end
  pop = sortrows(pop,m);
  
  for i=1:MaxPop
    m2 = fix(MaxScoutPop/2);
    % Use uniform random perturbations
    for j=1:m2
      for k=1:n
        scoutPop(j,k)  = pop(i,k) + (XHi(k) - XLow(k)) * StepFactor(k) .* (2*rand-1);
        if scoutPop(j,k) < XLow(k) || scoutPop(j,k) > XHi(k)
          scoutPop(j,k) = XLow(k) + (XHi(k) - XLow(k)) * rand;
        end
        
      end
      scoutPop(j,m) = fx(scoutPop(j,1:n));
    end
    % Use normal random perturbations
    for j=m2+1:MaxScoutPop
      for k=1:n
        scoutPop(j,k) = normrand(pop(i,k), (XHi(k) - XLow(k))/3 * StepFactor(k));
        if scoutPop(j,k) < XLow(k) || scoutPop(j,k) > XHi(k)
          scoutPop(j,k) = XLow(k) + (XHi(k) - XLow(k)) * rand;
        end
        
      end
      scoutPop(j,m) = fx(scoutPop(j,1:n));
    end
    pop = [pop; scoutPop];
    pop = sortrows(pop,m);
    pop = pop(1:MaxPop,:);
  end
  
  if bTrace && mod(iter, tpc) == 0
    fprintf('%f %f ', pop(1,1), pop(1,2));
    fprintf('%20.18e\nstep_factor(1)=%e, step_factor(2)=%e,\n', pop(1,m), StepFactor(1), StepFactor(2));
  end
end

bestX = pop(1,1:n);
bestFx = pop(1,m);
end

function y = normrand(mean,sigma)
  y = mean + sigma * randn(1,1);
end